/********************************************************************
 KWin - the KDE window manager
 This file is part of the KDE project.

Copyright (C) 2020 Chris Holland <zrenfire@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/

import QtQuick 2.0
import QtQuick.Layouts 1.3
import org.kde.plasma.core 2.0 as PlasmaCore
import org.kde.plasma.components 2.0 as PlasmaComponents
import QtQuick.Controls 2.5
import QtGraphicalEffects 1.0
import org.kde.kquickcontrolsaddons 2.0
import org.ukui.kwin 2.0 as KWin

// https://techbase.kde.org/Development/Tutorials/KWin/WindowSwitcher
// https://github.com/KDE/kwin/blob/master/tabbox/switcheritem.h
KWin.Switcher {
    id: tabBox
    currentIndex: thumbnailGridView.currentIndex

    PlasmaCore.Dialog {
        id: dialog
        location: PlasmaCore.Types.Floating
        visible: tabBox.visible
        backgroundHints: PlasmaCore.Dialog.NoBackground
        flags: Qt.X11BypassWindowManagerHint
        x: tabBox.screenGeometry.x + tabBox.screenGeometry.width * 0.5 - dialogMainItem.width * 0.5
        y: tabBox.screenGeometry.y + tabBox.screenGeometry.height * 0.5 - dialogMainItem.height * 0.5

        mainItem: Rectangle {
            id: dialogMainItem
            focus: true
            color: (tabBox.theme === "ukui-white" || tabBox.theme === "ukui" || tabBox.theme === "ukui-light")? Qt.rgba(245/255, 245/255, 245/255, 0.45) : Qt.rgba(35/255, 36/255, 38/255, 0.45)

            property int maxWidth: tabBox.screenGeometry.width
            property int maxHeight: tabBox.screenGeometry.height * 0.8
            property int maxGridColumnsByWidth: Math.floor(maxWidth / thumbnailGridView.cellWidth)
            property var scaleFactor : Math.min(tabBox.screenGeometry.width, tabBox.screenGeometry.height) / 1080

            property int gridColumns: maxGridColumnsByWidth
            property int gridRows: Math.ceil(thumbnailGridView.count / gridColumns)
            property int optimalWidth: thumbnailGridView.cellWidth * gridColumns
            property int optimalHeight: thumbnailGridView.cellHeight * gridRows
            property int gridViewMargin: 40 * scaleFactor
            width: maxWidth
            height: Math.min(optimalHeight + 2 * gridViewMargin, maxHeight)
            clip: true

            GridView {
                id: thumbnailGridView
                anchors.fill: parent
                anchors.bottomMargin: dialogMainItem.gridViewMargin
                anchors.topMargin: dialogMainItem.gridViewMargin
                anchors.leftMargin: count > dialogMainItem.maxGridColumnsByWidth ? (dialogMainItem.width-dialogMainItem.maxGridColumnsByWidth * cellWidth) / 2 :
                                                                                   (dialogMainItem.width-count * cellWidth) / 2
                anchors.rightMargin: count > dialogMainItem.maxGridColumnsByWidth ? (dialogMainItem.width-dialogMainItem.maxGridColumnsByWidth * cellWidth) / 2 :
                                                                                    (dialogMainItem.width-count * cellWidth) / 2
                model: tabBox.model
                property int iconSize: 24 * dialogMainItem.scaleFactor
                property int captionRowHeight: 33 * dialogMainItem.scaleFactor * units.devicePixelRatio
                property int thumbnailWidth: (tabBox.screenGeometry.width - 2 * dialogMainItem.gridViewMargin) / 4
                property int thumbnailHeight: tabBox.screenGeometry.height / 22 * 5
                property int spaceWidth: 2 * dialogMainItem.scaleFactor
                property int outerBorderWidth: 4 * dialogMainItem.scaleFactor
                property int innerBorderWidth: 2 * dialogMainItem.scaleFactor
                property int appRadius: 8 * dialogMainItem.scaleFactor
                property int thumbnailOuterRadius: tabBox.isOpenGL ? appRadius + outerBorderWidth + innerBorderWidth + 2 * spaceWidth : 0
                property int thumbnailInterRadius: tabBox.isOpenGL ? appRadius +innerBorderWidth + spaceWidth : 0
                cellWidth: 2 * (innerBorderWidth + outerBorderWidth) + 4 * spaceWidth + thumbnailWidth
                cellHeight: 2 * (innerBorderWidth + outerBorderWidth) + 5 * spaceWidth + captionRowHeight + thumbnailHeight
                keyNavigationWraps: true

                delegate: Rectangle {
                    id: thumbnailGridItem
                    width: thumbnailGridView.cellWidth
                    height: thumbnailGridView.cellHeight
                    color: "transparent"
                    border.color: (tabBox.theme === "ukui-white" || tabBox.theme === "ukui" || tabBox.theme === "ukui-light") ? "#262626" : "#FFFFFF"
                    border.width: index == thumbnailGridView.currentIndex ? thumbnailGridView.outerBorderWidth : 0
                    // highlight在快速绘制的时候会出现绘制的坐标偏移的问题，故不使用
                    radius: thumbnailGridView.thumbnailOuterRadius

                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            thumbnailGridItem.select();
                        }
                        hoverEnabled: true

                        onEntered: {
                            thumbnailBorderItem.border.width = thumbnailGridView.innerBorderWidth;
                        }

                        onExited: {
                            thumbnailBorderItem.border.width = 0;
                        }
                    }
                    function select() {
                        thumbnailGridView.currentIndex = index;
                        thumbnailGridView.currentIndexChanged(thumbnailGridView.currentIndex);
                        tabBox.selectWindow();
                    }

                    function getThumbnailSize()
                    {
                        var thumbHeight = thumbnailGridView.thumbnailHeight
                        var max_width = thumbnailGridView.thumbnailWidth
                        var clientWidth = model.width
                        var clientHeight = model.height
                        var thumbWidth = Math.floor(thumbHeight * clientWidth / clientHeight)
                        if (thumbWidth > max_width) {
                            thumbWidth = max_width
                            thumbHeight = Math.floor(thumbWidth * clientHeight / clientWidth)
                        }
                        return [thumbWidth, thumbHeight]
                    }

                    ColumnLayout {
                        spacing: 0
                        anchors.centerIn : parent
                        height: parent.height - 2 * (thumbnailGridView.outerBorderWidth + thumbnailGridView.spaceWidth)
                        width: parent.width - 2 * (thumbnailGridView.outerBorderWidth + thumbnailGridView.spaceWidth)
                        x: thumbnailGridView.outerBorderWidth + thumbnailGridView.spaceWidth
                        y: thumbnailGridView.outerBorderWidth + thumbnailGridView.spaceWidth
                        RowLayout {
                            id: captionRow
                            spacing: 2 * thumbnailGridView.spaceWidth
                            Layout.fillHeight: true

                            QIconItem {
                                id: iconItem
                                Layout.minimumHeight: thumbnailGridView.iconSize
                                Layout.minimumWidth: thumbnailGridView.iconSize
                                Layout.maximumHeight: Layout.minimumHeight
                                Layout.maximumWidth: Layout.minimumWidth
                                icon: model.icon
                                state: index == thumbnailGridView.currentIndex ? QIconItem.ActiveState : QIconItem.DefaultState
                            }

                            Label {
                                id: label
                                verticalAlignment: Qt.AlignVCenter
                                Layout.fillWidth: true
                                color: (tabBox.theme === "ukui-white" || tabBox.theme === "ukui" || tabBox.theme === "ukui-light") ? "#000000" : "#FFFFFF"
                                text: model.caption
                                elide: Text.ElideRight
                            }

                            //label高度受字号控制，此Rectangle起占位作用，防止字号改变时缩略图位置改变
                            Rectangle {
                                width: 1
                                height: thumbnailGridView.captionRowHeight
                                color: "transparent"
                            }
                        }

                        // KWin.ThumbnailItem needs a container
                        // otherwise it will be drawn the same size as the parent ColumnLayout
                        Rectangle {
                            color: "transparent"
                            Layout.fillHeight: true
                            Layout.fillWidth: true
                            Rectangle {
                                id: thumbnailBorderItem
                                x: (parent.width - thumbnailBorderItem.width) / 2
                                y: thumbnailGridView.spaceWidth
                                width: getThumbnailSize()[0] + 2 * (thumbnailGridView.innerBorderWidth + thumbnailGridView.spaceWidth)
                                height: thumbnailGridView.thumbnailHeight + 2 * (thumbnailGridView.innerBorderWidth + thumbnailGridView.spaceWidth)
                                color: "transparent"
                                border.color: (tabBox.theme === "ukui-white" || tabBox.theme === "ukui" || tabBox.theme === "ukui-light") ? "#262626" : "#FFFFFF"
                                border.width: 0
                                radius: thumbnailGridView.thumbnailInterRadius
                                Rectangle {
                                    id: thumbnailBackItem
                                    width: thumbnailBorderItem.width - 2 * (thumbnailGridView.innerBorderWidth + thumbnailGridView.spaceWidth)
                                    height: thumbnailGridView.thumbnailHeight
                                    anchors.centerIn: parent
                                    color: "transparent"
                                    radius: tabBox.isOpenGL ? thumbnailGridView.appRadius : 0
                                    clip: true
                                    layer.enabled: tabBox.isOpenGL ? true : false
                                    layer.effect: OpacityMask {
                                        maskSource : Rectangle {
                                            width: thumbnailBackItem.width
                                            height: thumbnailBackItem.height
                                            radius : tabBox.isOpenGL ? thumbnailGridView.appRadius : 0
                                        }
                                    }

                                    KWin.ThumbnailItem {
                                        id: thumbnailItem
                                        anchors.centerIn: parent
                                        width: getThumbnailSize()[0]
                                        height: getThumbnailSize()[1]
                                        wId: windowId
                                        clip: true
                                    }
                                }
                            }
                        }
                    }
                } // GridView.delegate

                Connections {
                    target: tabBox
                    onCurrentIndexChanged: {
                        thumbnailGridView.currentIndex = tabBox.currentIndex;
                    }
                }
            } // GridView

            Keys.onPressed: {
                if (event.key == Qt.Key_Left) {
                    thumbnailGridView.moveCurrentIndexLeft();
                } else if (event.key == Qt.Key_Right) {
                    thumbnailGridView.moveCurrentIndexRight();
                } else if (event.key == Qt.Key_Up) {
                    thumbnailGridView.moveCurrentIndexUp();
                } else if (event.key == Qt.Key_Down) {
                    thumbnailGridView.moveCurrentIndexDown();
                } else {
                    return;
                }

                thumbnailGridView.currentIndexChanged(thumbnailGridView.currentIndex);
            }
        } // Dialog.mainItem
    } // Dialog
}
